<?php
/**
 * Add-On for UCM to Manage Digital Ocean Cloud Storage
 * Package: Ultimate Media On The Cloud
 * Plugin URI: https://www.phprockets.com/digital-ocean-addon-ucm
 * Date: 08-Aug-2019
 */
class PhpRockets_UCM_DigitalOcean_AddOn extends PhpRockets_UCM_Addons
{
    //Hard set to override current active account
    public static $account;
    public static $is_migration = false;
    public static $is_migration_remove_file = false;

    /* Alias key for using add hook,db,etc.. */
    public static $addon_alias_key = 'digitalocean';

    /**
     * AddOn Information
     **/
    public $labels = [
        'namespace' => __CLASS__,
        'title' => 'Digital Ocean',
        'uri' => 'digitaloceanspaces.com',
        'url' => [
            '-dg-save' => 'saveSettings',
            '-dg-connect' => 'connect',
            '-retrieve-dg-credentials' => 'retrieveCredentials'
        ]
    ];

    /**
     * Check if UCM plugin is LITE or PRO
     * @return bool
     */
    private static function _isProBuild()
    {
        return UCM_PLUGIN_TYPE === 'PRO';
    }
    /**
     * Register AddOn handler
     *
     * @return void
     */
    public function register() {
        apply_filters('ucm_register_addons_vendor', 'ucm-addons-'. self::$addon_alias_key .'/vendor/sociallydev/spaces-api/spaces.php', 'external');
        add_filter('ucm_'. self::$addon_alias_key .'_upload_media', [$this, 'doPushAttachment'], $this::$configs->default_order , 1);
        if (get_option(self::$configs->plugin_db_prefix .'advanced_delete_cloud_file')) {
            add_filter('ucm_'. self::$addon_alias_key .'_cloud_remove_file', [$this, 'doRemoveAttachmentMedia'], $this::$configs->default_order, 1);
        }
        add_filter('ucm_'. self::$addon_alias_key .'_cloud_file_url', [$this, 'getAdapterStorageUrl'], $this::$configs->default_order, 2);
    }

    /**
     * Register EnqueueScript for backend settings page
     *
     * @return void
     */
    public function registerEnqueueScript()
    {
        wp_enqueue_script('phprockets-ucm-digital-ocean', dirname(plugin_dir_url(__FILE__)) .'/js/ucm-'. self::$addon_alias_key . self::$configs->enqueue_assets_suffix .'.js', ['jquery', 'phprockets-ucm-settings']);
        $vars = [];
        foreach ($this->labels['url'] as $url => $callback) {
            $vars[$callback] = admin_url('admin-ajax.php?action='. $this::$configs->plugin_url_prefix . $url);
        }
        wp_localize_script( 'phprockets-ucm-settings', 'phprockets_ucm_dg', $vars);
        $ucm_l10n = [
            '_require_all_fields' => __('Please complete all fields.', 'ultimate-media-on-the-cloud'),
            '_required_master_password' => __('Please enter master password.', 'ultimate-media-on-the-cloud')
        ];
        wp_localize_script( 'phprockets-ucm-settings', 'phprockets_dg_l10n', $ucm_l10n);
    }

    /**
     * {@inheritDoc}
     * @throws SpacesAPIException
     */
    public function initClient ($args) {
        return new SpacesConnect($args['app_key'], $args['app_secret'], $args['bucket'], $args['region']);
    }

    /**
     * Handle saving setting in backend
     *
     * @return void
     * @throws Exception
     */
    public static function saveSettings()
    {
        if (self::isPost()) {
            $data = self::getPost('data');

            /* Handle in Pro built of UCM */
            if (self::_isProBuild()) {
                self::handlePremiumForm($data);
                wp_die();
            }

            /* Perform validation inputs */
            $validation = new Validation();
            $validation->validation_rules([
                'app_key' => 'required',
                'app_secret' => 'required',
                'space_name' => 'required',
                'region' => 'required'
            ]);
            $validated = $validation->run($data);
            if ($validated === false) {
                $error_messages = $validation->get_errors_array();
                wp_send_json_error(['message' => implode('<br>', $error_messages)]);
                wp_die();
            }

            /* Perform an unit test*/
            $unit_test = self::doUnitSettingsTest($data);
            if (is_wp_error($unit_test)) {
                wp_send_json_error(['message' => $unit_test->get_error_message('exception')]);
                wp_die();
            }

            $account = PhpRockets_Model_Accounts::query([
                'conditions' => [
                    'storage_adapter' => self::$addon_alias_key,
                    'is_default' => 1
                ],
                'limit' => 1
            ]);

            $data['bucket'] = $data['space_name'];
            unset($data['space_name']);

            if (!$account) {
                $create_account = PhpRockets_Model_Accounts::create([
                    'storage_adapter' => self::$addon_alias_key,
                    'name' => 'Default Digital Ocean Account',
                    'addon_class' => 'PhpRockets_UCM_DigitalOcean_AddOn',
                    'is_default' => 1,
                    'value' => serialize($data),
                    'created_at' => current_time('mysql', 1),
                    'updated_at' => current_time('mysql', 1)
                ]);

                if ($create_account) {
                    wp_send_json_success(['message' => __('Account information updated.', 'ultimate-media-on-the-cloud')]);
                } else {
                    wp_send_json_error(['message' => __('An error occurred while updating account. Try again later!', 'ultimate-media-on-the-cloud')]);
                }
            } else {
                $update_account = PhpRockets_Model_Accounts::update($account['id'], [
                    'value' => serialize($data),
                    'updated_at' => current_time('mysql', 1)
                ]);

                if ($update_account) {
                    wp_send_json_success(['message' => __('Account information updated.', 'ultimate-media-on-the-cloud')]);
                } else {
                    wp_send_json_error(['message' => __('An error occurred while updating account. Try again later!', 'ultimate-media-on-the-cloud')]);
                }
            }

            wp_send_json_success(['message' => __('Credentials are corrected!', 'ultimate-media-on-the-cloud')]);
        } else {
            wp_send_json_error(['message' => __('Invalid request', 'ultimate-media-on-the-cloud')]);
        }

        wp_die();
    }

    /**
     * @param $data
     * @return bool|void
     * @throws Exception
     */
    public static function handlePremiumForm($data)
    {
        $validation = new Validation();
        $validation->validation_rules([
            'account_name' => 'required',
            'app_key' => 'required',
            'app_secret' => 'required',
            'cloud_path' => 'required',
            'space_name' => 'required',
            'region' => 'required'
        ]);

        $validated = $validation->run($data);
        if ($validated === false) {
            $error_messages = $validation->get_errors_array();
            wp_send_json_error(['message' => implode('<br>', $error_messages)]);
            return false;
        }

        /* Perform an unit test*/
        $unit_test = self::doUnitSettingsTest($data);
        if (is_wp_error($unit_test)) {
            wp_send_json_error(['message' => $unit_test->get_error_message('exception')]);
            return false;
        }

        $id = isset($data['id']) ? $data['id'] : null;
        $data['bucket'] = $data['space_name'];
        unset($data['space_name']);

        if ($id) {
            $account = PhpRockets_Model_Accounts::query([
                'conditions' => [
                    'storage_adapter' => self::$addon_alias_key,
                    'id' => $id
                ]
            ]);
            if (!$account) {
                wp_send_json_error(['message' => __('Account is not found!', 'ultimate-media-on-the-cloud')]);
                wp_die();
            }

            /* There is no change on credential (masked as *) */
            $account_data = unserialize($account['value']);
            if ($data['app_key'] === '**********') {
                $data['app_key'] = $account_data['app_key'];
            }
            if ($data['app_secret'] === '**********') {
                $data['app_secret'] = $account_data['app_secret'];
            }

            /* Check if existed account with same name */
            $check_account_exist = PhpRockets_Model_Accounts::query([
                'conditions' => [
                    'storage_adapter' => self::$addon_alias_key,
                    'name' => $data['account_name'],
                    'id' => [
                        'value' => $id,
                        'operator' => '<>'
                    ]
                ],
                'limit' => 1
            ]);

            if ($check_account_exist) {
                wp_send_json_error(['message' => 'There is another account with same name in Database. Please choose another one!']);
            } else {
                $account_data = [
                    'name' => $data['account_name'],
                    'value' => serialize($data),
                    'updated_at' => current_time('mysql', 1)
                ];
                $update_account = PhpRockets_Model_Accounts::update($id, $account_data);
                if ($update_account) {
                    wp_send_json_success(['message' => __('Digital Ocean account updated.', 'ultimate-media-on-the-cloud')]);
                } else {
                    wp_send_json_error(['message' => __('An error occurred while updating account. Try again later!', 'ultimate-media-on-the-cloud')]);
                }
            }
        } else {
            $check_account_exist = PhpRockets_Model_Accounts::query([
                'conditions' => [
                    'storage_adapter' => self::$addon_alias_key,
                    'name' => $data['account_name']
                ],
                'limit' => 1
            ]);

            if ($check_account_exist) {
                wp_send_json_error(['message' => 'There is another account with same name in Database. Please choose another one!']);
            } else {
                $have_account = PhpRockets_Model_Accounts::query([
                    'conditions' => [
                        'storage_adapter' => self::$addon_alias_key
                    ],
                    'limit' => 1
                ]);
                $is_default = $have_account ? 0 : 1;

                $account_data = [
                    'storage_adapter' => self::$addon_alias_key,
                    'name' => $data['account_name'],
                    'is_default' => $is_default,
                    'addon_class' => 'PhpRockets_UCM_DigitalOcean_AddOn',
                    'value' => serialize($data),
                    'created_at' => current_time('mysql', 1),
                    'updated_at' => current_time('mysql', 1)
                ];

                $create_account = PhpRockets_Model_Accounts::create($account_data);
                if ($create_account) {
                    wp_send_json_success(['message' => __('Digital Ocean account created.', 'ultimate-media-on-the-cloud')]);
                } else {
                    wp_send_json_error(['message' => __('An error occurred while creating account. Try again later!', 'ultimate-media-on-the-cloud')]);
                }
            }
        }
    }

    /**
     * Test a config is success connect to Digital Ocean or not
     *
     * @return void
     * @throws Exception
     */
    public static function connect()
    {
        if (self::isPost()) {
            $data = self::getPost('data');
            /* Perform validation inputs */
            $validation = new Validation();
            $validation->validation_rules([
                'app_key' => 'required',
                'app_secret' => 'required',
                'space_name' => 'required',
                'region' => 'required'
            ]);
            $validated = $validation->run($data);
            if ($validated === false) {
                $error_messages = $validation->get_errors_array();
                wp_send_json_error(['message' => implode('<br>', $error_messages)]);
                wp_die();
            }

            /* Perform an unit test*/
            $unit_test = self::doUnitSettingsTest($data);
            if (is_wp_error($unit_test)) {
                wp_send_json_error(['message' => $unit_test->get_error_message('exception')]);
                wp_die();
            }

            wp_send_json_success(['message' => __('Credentials are corrected!', 'ultimate-media-on-the-cloud')]);
        } else {
            wp_send_json_error(['message' => __('Invalid request', 'ultimate-media-on-the-cloud')]);
        }

        wp_die();
    }

    /**
     * Perform an UnitTest if Configuration is corrected
     *
     * @param $data
     * @return bool|WP_Error
     */
    private static function doUnitSettingsTest($data)
    {
        /* If id of account is submitted */
        $id = isset($data['id']) ? $data['id'] : null;
        $account = PhpRockets_Model_Accounts::query([
            'conditions' => [
                'storage_adapter' => self::$addon_alias_key,
                'id' => $id
            ]
        ]);
        if ($account) {
            $account_data = unserialize($account['value']);
            $app_key = $data['app_key'] === '**********' ? $account_data['app_key'] : $data['app_key'];
            $app_secret = $data['app_secret'] === '**********' ? $account_data['app_secret'] : $data['app_secret'];
        } else {
            $app_key = $data['app_key'];
            $app_secret = $data['app_secret'];
        }

        /* Init Digital Ocean client object*/
        try {
            $space = new SpacesConnect($app_key, $app_secret, $data['space_name'], $data['region']);
        } catch (SpacesAPIException $e) {
            $WP_Error = new WP_Error();
            $WP_Error->add('exception', __('The credentials are not valid', 'ultimate-media-on-the-cloud'));
            return $WP_Error;
        }

        $space_names = $space->ListSpaces();
        if (is_wp_error($space_names)) {
            return $space_names;
        }

        $file = ULTIMATE_MEDIA_PLG_DIR .'/assets/test.txt';
        $mine_type = 'application/octet-stream';

        /* Perform a quick test */
        try {
            $space->UploadFile($file, 'public', 'test.txt', $mine_type);
        } catch (\Exception $e) {
            $WP_Error = new WP_Error();
            $WP_Error->add('exception', 'The credentials are not valid');
            return $WP_Error;
        }

        $space->DeleteObject('test.txt');
        return true;
    }

    /**
     * Generate form setting page
     *
     * @return mixed
     */
    public function buildForm() {
        $errors = [];
        $messages = [];
        $account = PhpRockets_Model_Accounts::query([
            'conditions' => [
                'storage_adapter' => self::$addon_alias_key,
                'is_default' => 1
            ],
            'limit' => 1
        ]);

        $account_data = [];
        if ($account) {
            $account_data = unserialize($account['value']);
            $messages[] = __('Digital Ocean default account has been set.', 'ultimate-media-on-the-cloud');
        }
        if (!$account) {
            if (self::_isProBuild()) {
                $errors[] = __('You have no Digital Ocean default account. Please check <a href="'. admin_url() .'admin.php?page='. self::$configs->getMenuSlug('account_menu') .'&ucm-tab='. self::$addon_alias_key .'">Digital Ocean Page</a>', 'ultimate-media-on-the-cloud');
            } else {
                $errors[] = __('Set up Digital Ocean Account by the form below', 'ultimate-media-on-the-cloud');
            }
        }

        if (self::_isProBuild()) {
            $form = [
                'div' => [
                    'class' => 'account-panel-body',
                    'id' => 'phprockets-ucm-'. self::$addon_alias_key .'-addon'
                ],
                'attr' => [
                    'id' => 'frm-ucm-'. self::$addon_alias_key,
                    'onSubmit' => 'return false;'
                ],
                'fields' => [],
                'submit' => [
                    'label' => __('Manage Accounts', 'ultimate-media-on-the-cloud'),
                    'attr' => [
                        'href' => admin_url() .'admin.php?page='. self::$configs->getMenuSlug('account_menu'),
                        'class' => 'button is-info'
                    ]
                ]
            ];

            if ($account) {
                $form['cancel'] = [
                    'label' => __('Edit this account', 'ultimate-media-on-the-cloud'),
                    'attr' => [
                        'href' => admin_url() .'admin.php?page='. self::$configs->getMenuSlug('account_menu') .'&ucm-action=edit&type='. self::$addon_alias_key .'&id='. $account['id'],
                        'class' => 'button is-info'
                    ]
                ];
                $form['fields'][] = [
                    'label' => __('Name', 'ultimate-media-on-the-cloud'),
                    'type'  => 'text',
                    'icon' => 'dashicons dashicons-post-status',
                    'attr' => [
                        'name' => 'data[name]',
                        'value' => $account ? $account['name'] : '',
                        'placeholder' => __('Name', 'ultimate-media-on-the-cloud'),
                        'class' => 'input',
                        'id' => 'dg-app-key',
                        'disabled' => 'disabled'
                    ]
                ];

                $form['fields'][] = [
                    'label' => __('Space Name', 'ultimate-media-on-the-cloud'),
                    'type' => 'text',
                    'icon' => 'dashicons dashicons-archive',
                    'attr' => [
                        'name' => 'data[space_name]',
                        'value' => $account_data ? $account_data['bucket'] : '',
                        'placeholder' => __('Space name', 'ultimate-media-on-the-cloud'),
                        'class' => 'input',
                        'id' => 'dg-space-name',
                        'disabled' => 'disabled'
                    ],
                    'help-text' => __('Fill the target space name.', 'ultimate-media-on-the-cloud')
                ];

                $form['fields'][] = [
                    'label' => __('Cloud Path', 'ultimate-media-on-the-cloud'),
                    'type' => 'text',
                    'icon' => 'dashicons dashicons-archive',
                    'attr' => [
                        'name' => 'data[cloud_path]',
                        'value' => $account_data ? $account_data['cloud_path'] : '/',
                        'placeholder' => __('Cloud Path', 'ultimate-media-on-the-cloud'),
                        'class' => 'input',
                        'id' => 'dg-cloud-path',
                        'disabled' => 'disabled'
                    ],
                    'help-text' => __('Target folder at cloud storage you want to save media to.', 'ultimate-media-on-the-cloud')
                ];

                $form['fields'][] = [
                    'label' => __('Region', 'ultimate-media-on-the-cloud'),
                    'type' => 'select',
                    'value' => array_merge(['' => '-Choose Region-'], $this->getAvailableRegions()),
                    'selected' => $account_data ? $account_data['region'] : '',
                    'icon' => 'dashicons dashicons-flag',
                    'attr' => [
                        'name' => 'data[region]',
                        'id' => 'dg-app-region',
                        'disabled' => 'disabled'
                    ],
                    'help-text' => __('Important: Please choose the correct region which you registered at Digital Ocean.', 'ultimate-media-on-the-cloud')
                ];

                $form['fields'][] = [
                    'label' => __('Enable CDN?', 'ultimate-media-on-the-cloud'),
                    'type' => 'select',
                    'value' => [0 => 'No', 1 => 'Yes'],
                    'selected' => $account_data ? $account_data['enable_cdn'] : '',
                    'icon' => 'dashicons dashicons-flag',
                    'attr' => [
                        'name' => 'data[enable_cdn]',
                        'id' => 'dg-enable-cdn',
                        'disabled' => 'disabled'
                    ],
                    'help-text' => __('Is the CDN enabled to your space?', 'ultimate-media-on-the-cloud')
                ];
            }
        } else {
            $form = [
                'div' => [
                    'class' => 'account-panel-body',
                    'id' => 'phprockets-ucm-'. self::$addon_alias_key .'-addon'
                ],
                'attr' => [
                    'id' => 'frm-ucm-'. self::$addon_alias_key,
                    'onSubmit' => 'return false;'
                ],
                'fields' => [],
                'submit' => [
                    'label' => __('Save Settings', 'ultimate-media-on-the-cloud'),
                    'attr' => [
                        'href' => 'javascript:;',
                        'id' => 'btn-'. self::$addon_alias_key .'-setting',
                        'class' => 'button is-info'
                    ]
                ]
            ];

            $form['fields'][] = [
                'label' => __('App Key', 'ultimate-media-on-the-cloud'),
                'type'  => 'text',
                'icon' => 'dashicons dashicons-post-status',
                'attr' => [
                    'name' => 'data[app_key]',
                    'value' => $account_data ? $account_data['app_key'] : '',
                    'placeholder' => __('App Key', 'ultimate-media-on-the-cloud'),
                    'class' => 'input',
                    'id' => 'dg-app-key'
                ]
            ];

            $form['fields'][] = [
                'label' => 'App Secret',
                'type' => 'text',
                'icon' => 'dashicons dashicons-lock',
                'attr' => [
                    'name' => 'data[app_secret]',
                    'value' => $account_data ? $account_data['app_secret'] : '',
                    'placeholder' => __('App Secret Key', 'ultimate-media-on-the-cloud'),
                    'class' => 'input',
                    'id' => 'dg-app-secret',
                ],
                'help-text' => __('Login to', 'ultimate-media-on-the-cloud') .
                    ' <a href="'. $this::$configs->getUcmConfig('digital_ocean_console') .'" class="button is-link is-small" target="_blank">'. __('Digital Ocean console', 'ultimate-media-on-the-cloud') .'</a>'
            ];

            $form['fields'][] = [
                'label' => __('Space Name', 'ultimate-media-on-the-cloud'),
                'type' => 'text',
                'icon' => 'dashicons dashicons-archive',
                'attr' => [
                    'name' => 'data[space_name]',
                    'value' => $account_data ? $account_data['bucket'] : '',
                    'placeholder' => __('Space name', 'ultimate-media-on-the-cloud'),
                    'class' => 'input',
                    'id' => 'dg-space-name',
                ],
                'help-text' => __('Fill the target space name.', 'ultimate-media-on-the-cloud')
            ];

            $form['fields'][] = [
                'label' => __('Cloud Path', 'ultimate-media-on-the-cloud'),
                'type' => 'text',
                'icon' => 'dashicons dashicons-archive',
                'attr' => [
                    'name' => 'data[cloud_path]',
                    'value' => $account_data ? $account_data['cloud_path'] : '/',
                    'placeholder' => __('Cloud Path', 'ultimate-media-on-the-cloud'),
                    'class' => 'input',
                    'id' => 'dg-cloud-path',
                ],
                'help-text' => __('Target folder at cloud storage you want to save media to.', 'ultimate-media-on-the-cloud')
            ];

            $form['fields'][] = [
                'label' => __('Region', 'ultimate-media-on-the-cloud'),
                'type' => 'select',
                'value' => array_merge(['' => '-Choose Region-'], $this->getAvailableRegions()),
                'selected' => $account_data ? $account_data['region'] : '',
                'icon' => 'dashicons dashicons-flag',
                'attr' => [
                    'name' => 'data[region]',
                    'id' => 'dg-app-region',
                ],
                'help-text' => __('Important: Please choose the correct region which you registered at Digital Ocean.', 'ultimate-media-on-the-cloud')
            ];


            $form['fields'][] = [
                'label' => __('Connect', 'ultimate-media-on-the-cloud'),
                'type' => 'anchor',
                'attr' => [
                    'href' => 'javascript:;',
                    'icon' => 'dashicons dashicons-plugins-checked icon-button',
                    'class' => 'button is-link',
                    'id' => 'btn-'. self::$addon_alias_key .'-connect'
                ],
                'help-text' => __('Click Connect button above to perform a quick test.', 'ultimate-media-on-the-cloud')
            ];

            $form['fields'][] = [
                'label' => __('Enable CDN?', 'ultimate-media-on-the-cloud'),
                'type' => 'select',
                'value' => [0 => 'No', 1 => 'Yes'],
                'selected' => 0,
                'icon' => 'dashicons dashicons-flag',
                'attr' => [
                    'name' => 'data[enable_cdn]',
                    'id' => 'dg-enable-cdn',
                    'disabled' => 'disabled'
                ],
                'help-text' => __('Is the CDN enabled to your space? <strong>This option available at PRO version</strong>', 'ultimate-media-on-the-cloud')
            ];
        }

        return $this::renderTemplate('common/_form', ['form' => $form, 'errors' => $errors, 'messages' => $messages], false);
    }

    /**
     * Register ajax URLs for AddOn
     *
     * @return mixed
     */
    public function registerAjaxUrlHook() {
        $ajax = [];
        foreach ($this->labels['url'] as $url => $callback) {
            $ajax['wp_ajax_'. $this::$configs->plugin_url_prefix . $url] = $callback;
        }

        return $ajax;
    }

    /**
     * Handle Push/Upload file to Cloud Storage Server
     *
     * @param $data
     * @param $post_id
     * @return mixed
     * @throws SpacesAPIException
     */
    public function doPushAttachment($data) {
        if (self::_isProBuild()) {
            $account = self::$account ?: $this->activeAdapter;
            $account_config = unserialize($account['value']);
        } else {
            $account_config = unserialize($this->activeAdapter['value']);
        }

        $space = new SpacesConnect($account_config['app_key'], $account_config['app_secret'], $account_config['bucket'], $account_config['region']);
        $wp_upload_dir = wp_upload_dir();
        $file_upload_dir = explode('/', $data['file']);
        unset($file_upload_dir[count($file_upload_dir) - 1]);
        $file_upload_dir = implode('/', $file_upload_dir);

        $files = [str_replace($file_upload_dir .'/', '', $data['file'])]; //Main file
        if ($data['sizes']) {
            foreach ($data['sizes'] as $generated_file) {
                $files[] = $generated_file['file'];
            }
        }
        $mine_type = isset($data['sizes']['thumbnail']['mime-type']) ? $data['sizes']['thumbnail']['mime-type'] : '';
        $root = $this->correctUploadRootDirForPush($account_config['cloud_path']);

        foreach ($files as $file) {
            if (file_exists($wp_upload_dir['basedir'] . "/{$file_upload_dir}/{$file}")) {
                $result = $space->UploadFile($wp_upload_dir['basedir'] . "/{$file_upload_dir}/{$file}", 'public', $root . $file_upload_dir .'/' . $file, $mine_type);
                if (is_wp_error($result)) {
                    return $result;
                }
            } else {
                return new WP_Error('exception', "File: /{$file_upload_dir}/{$file} does not exist.");
            }
        }

        if (self::_isProBuild()) {
            /* Remove local file if option is Set TRUE */
            if (self::$is_migration && self::$is_migration_remove_file) {
                apply_filters('ucm_host_cleanup', $data);
            } elseif (!self::$is_migration && !get_option(self::$configs->plugin_db_prefix .'option_keep_copy')) {
                apply_filters('ucm_host_cleanup', $data);
            }
        } else if (!get_option(self::$configs->plugin_db_prefix .'option_keep_copy')) {
            apply_filters('ucm_host_cleanup', $data);
        }

        return true;
    }

    /**
     * Handle remove file on cloud
     *
     * @param $post_id
     * @return mixed
     */
    public function doRemoveAttachmentMedia ($post_id) {
        $data = wp_get_attachment_metadata($post_id);
        $storage_metadata = get_post_meta($post_id, '_ucm_storage_metadata', true);
        if ($data && $storage_metadata) {
            $storage_metadata = unserialize($storage_metadata);
            if ($storage_metadata['account_id']) {
                $adapter_account = PhpRockets_Model_Accounts::query([
                    'conditions' => [
                        'id' => $storage_metadata['account_id']
                    ]
                ]);
                if (!$adapter_account) {
                    return false;
                }

                $account_configs = unserialize($adapter_account['value']);
                try {
                    $space = new SpacesConnect($account_configs['app_key'], $account_configs['app_secret'], $account_configs['bucket'], $account_configs['region']);
                } catch (SpacesAPIException $e) {
                    return false;
                }

                $file_upload_dir = explode('/', $data['file']);
                unset($file_upload_dir[count($file_upload_dir) - 1]);
                $file_upload_dir = implode('/', $file_upload_dir);

                $files = [str_replace($file_upload_dir .'/', '', $data['file'])]; //Main file
                if ($data['sizes']) {
                    foreach ($data['sizes'] as $generated_file) {
                        $files[] = $generated_file['file'];
                    }
                }
                $root = $this->correctUploadRootDirForPush($storage_metadata['path']);

                $objects = [];
                foreach ($files as $file) {
                    $objects[] = [
                        'Key' => $root . $file_upload_dir .'/'. $file
                    ];
                }

                try {
                    foreach ($objects as $object) {
                        $space->DeleteObject($object['Key']);
                    }
                } catch (\Exception $e) {
                    return false;
                }

                return true;
            }
        }

        return true;
    }

    /**
     * Generate table for management account
     *
     * @return mixed
     */
    public static function renderManageAccountTable()
    {
        $storage_accounts = PhpRockets_Model_Accounts::query([
            'conditions' => [
                'storage_adapter' => self::$addon_alias_key
            ],
            'order_by' => 'id DESC'
        ]);

        return self::renderTemplate(dirname(plugin_dir_path(__FILE__)) . '/tpl/accounts', [
            'accounts' => $storage_accounts,
            'self_addon' => new self
        ], true, false, 'external');
    }

    /**
     * {@inheritDoc}
     */
    public static function renderCreateAccountPage($id)
    {
        $instance = new self();
        return self::renderTemplate(dirname(plugin_dir_path(__FILE__)) .'/tpl/create', [
            'form' => $instance->_buildAccountForm($id)
        ], true, false, 'external');
    }

    /**
     * Build the create/edit account form template
     *
     * @param integer $id
     * @return false|string
     */
    private function _buildAccountForm($id = null)
    {
        $errors = [];
        $messages = [];
        $account = $account_data = [];
        if ($id) {
            $account = PhpRockets_Model_Accounts::query([
                'conditions' => [
                    'storage_adapter' => self::$addon_alias_key,
                    'id' => $id
                ]
            ]);
            if ($account) {
                $account_data = unserialize($account['value']);
            }
        }

        $form = [
            'div' => [
                'class' => 'account-panel-body is-active',
                'id' => 'phprockets-ucm-'. self::$addon_alias_key .'-addon'
            ],
            'attr' => [
                'id' => 'frm-ucm-'. self::$addon_alias_key,
                'onSubmit' => 'return false;'
            ],
            'fields' => [],
            'submit' => [
                'label' => __($account_data ? 'Update' : 'Create', 'ultimate-media-on-the-cloud'),
                'attr' => [
                    'href' => 'javascript:;',
                    'id' => 'btn-'. self::$addon_alias_key .'-setting',
                    'class' => 'button is-info'
                ]
            ],
            'cancel' => [
                'label' => __('Back', 'ultimate-media-on-the-cloud'),
                'attr' => [
                    'href' => admin_url() .'admin.php?page='. self::$configs->getMenuSlug('account_menu'),
                    'class' => 'button is-danger'
                ]
            ]
        ];

        if ($account) {
            $form['fields'][] = [
                'type'  => 'hidden',
                'attr' => [
                    'name' => 'data[id]',
                    'value' => $account['id'],
                ]
            ];
        }

        $form['fields'][] = [
            'label' => __('Account name', 'ultimate-media-on-the-cloud'),
            'type'  => 'text',
            'icon' => 'dashicons dashicons-admin-users',
            'attr' => [
                'name' => 'data[account_name]',
                'value' => $account ? $account['name'] : '',
                'placeholder' => __('Choose an account name', 'ultimate-media-on-the-cloud'),
                'class' => 'input',
                'id' => 'account-name'
            ]
        ];

        /* Hide credentials if master password is set */
        $ucm_master_password = self::ucmGetOption('master_password');
        if (isset($account) && $account_data && $ucm_master_password) {
            $app_key = '**********';
            $app_secret = '**********';
            $form['fields'][] = [
                'label' => __('Retrieve Credentials', 'ultimate-media-on-the-cloud'),
                'type' => 'anchor',
                'div' => [
                    'class' => 'field columns',
                    'id' => 'div-retrieve-credentials'
                ],
                'attr' => [
                    'href' => 'javascript:;',
                    'icon' => 'dashicons dashicons-plugins-checked icon-button',
                    'class' => 'button is-primary',
                    'onclick' => "phpR_UCM.modal.show(jQuery('#modal-retrieve-credentials'));"
                ]
            ];

            /* Build the retrieve credentials form */
            $credential_form = [
                'div' => [
                    'class' => 'panel-body is-active',
                    'id' => 'ucm-retrieve-credentials'
                ],
                'attr' => [
                    'name' => 'ucm_form_retrieve_credentials',
                    'id' => 'ucm-form-retrieve-credentials',
                    'onSubmit' => 'return false;'
                ],
                'fields' => [],
                'submit' => [
                    'label' => __('Retrieve', 'ultimate-media-on-the-cloud'),
                    'attr' => [
                        'href' => 'javascript:;',
                        'id' => 'btn-dg-retrieve-credentials',
                        'data-account' =>  $account['id'],
                        'class' => 'button is-primary'
                    ]
                ]
            ];

            $credential_form['fields'][] = [
                'label' => 'Master Password',
                'type' => 'password',
                'icon' => 'dashicons dashicons-tag',
                'attr' => [
                    'name' => 'master_password',
                    'class' => 'input',
                    'id' => 'master-password',
                    'placeholder' => '******'
                ]
            ];

            /* Render the form template html */
            $credential_form = $this::renderTemplate('common/_form', ['form' => $credential_form], false);

            /* Build the retrieve credentials modal */
            $modal_retrieve_credentials = [
                'title' => __('Retrieve Credentials', 'ultimate-media-on-the-cloud'),
                'id' => 'modal-retrieve-credentials',
                'content' => $credential_form,
                'button_cancel' => [
                    'label' => __('Close', 'ultimate-media-on-the-cloud'),
                    'attr' => [
                        'class' => 'button btn-close-modal'
                    ]
                ]
            ];
            $modal_retrieve_credentials = $this::renderTemplate('common/modal', ['modal' => $modal_retrieve_credentials], false);
        } else {
            $app_key = isset($account) && $account_data ? $account_data['app_key'] : '';
            $app_secret = isset($account) && $account_data ? $account_data['app_secret'] : '';
        }

        $form['fields'][] = [
            'label' => __('App Key', 'ultimate-media-on-the-cloud'),
            'type'  => 'text',
            'icon' => 'dashicons dashicons-post-status',
            'attr' => [
                'name' => 'data[app_key]',
                'value' => $app_key,
                'placeholder' => __('App Key', 'ultimate-media-on-the-cloud'),
                'class' => 'input',
                'id' => 'dg-app-key'
            ]
        ];

        $form['fields'][] = [
            'label' => 'App Secret',
            'type' => 'text',
            'icon' => 'dashicons dashicons-lock',
            'attr' => [
                'name' => 'data[app_secret]',
                'value' => $app_secret,
                'placeholder' => __('App Secret Key', 'ultimate-media-on-the-cloud'),
                'class' => 'input',
                'id' => 'dg-app-secret',
            ],
            'help-text' => __('Login to', 'ultimate-media-on-the-cloud') .
                ' <a href="'. $this::$configs->getUcmConfig('digital_ocean_console') .'" class="button is-link is-small" target="_blank">'. __('Digital Ocean console', 'ultimate-media-on-the-cloud') .'</a>'
        ];

        $form['fields'][] = [
            'label' => __('Space Name', 'ultimate-media-on-the-cloud'),
            'type' => 'text',
            'icon' => 'dashicons dashicons-archive',
            'attr' => [
                'name' => 'data[space_name]',
                'value' => $account_data ? $account_data['bucket'] : '',
                'placeholder' => __('Space name', 'ultimate-media-on-the-cloud'),
                'class' => 'input',
                'id' => 'dg-space-name',
            ],
            'help-text' => __('Fill the target space name.', 'ultimate-media-on-the-cloud')
        ];

        $form['fields'][] = [
            'label' => __('Cloud Path', 'ultimate-media-on-the-cloud'),
            'type' => 'text',
            'icon' => 'dashicons dashicons-archive',
            'attr' => [
                'name' => 'data[cloud_path]',
                'value' => $account_data ? $account_data['cloud_path'] : '/',
                'placeholder' => __('Cloud Path', 'ultimate-media-on-the-cloud'),
                'class' => 'input',
                'id' => 'dg-cloud-path',
            ],
            'help-text' => __('Target folder at cloud storage you want to save media to.', 'ultimate-media-on-the-cloud')
        ];

        $form['fields'][] = [
            'label' => __('Region', 'ultimate-media-on-the-cloud'),
            'type' => 'select',
            'value' => array_merge(['' => '-Choose Region-'], $this->getAvailableRegions()),
            'selected' => $account_data ? $account_data['region'] : '',
            'icon' => 'dashicons dashicons-flag',
            'attr' => [
                'name' => 'data[region]',
                'id' => 'dg-app-region',
            ],
            'help-text' => __('Important: Please choose the correct region which you registered at Digital Ocean.', 'ultimate-media-on-the-cloud')
        ];

        $form['fields'][] = [
            'label' => __('Connect', 'ultimate-media-on-the-cloud'),
            'type' => 'anchor',
            'attr' => [
                'href' => 'javascript:;',
                'icon' => 'dashicons dashicons-plugins-checked icon-button',
                'class' => 'button is-link',
                'id' => 'btn-'. self::$addon_alias_key .'-connect'
            ],
            'help-text' => __('Click Connect button above to perform a quick test.', 'ultimate-media-on-the-cloud')
        ];

        $form['fields'][] = [
            'label' => __('Enable CDN?', 'ultimate-media-on-the-cloud'),
            'type' => 'select',
            'value' => [0 => 'No', 1 => 'Yes'],
            'selected' => $account_data ? (int)$account_data['enable_cdn'] : '',
            'icon' => 'dashicons dashicons-flag',
            'attr' => [
                'name' => 'data[enable_cdn]',
                'id' => 'dg-enable-cdn'
            ],
            'help-text' => __('Is the CDN enabled to your space?', 'ultimate-media-on-the-cloud')
        ];

        $template = $this::renderTemplate('common/_form', ['form' => $form, 'errors' => $errors, 'messages' => $messages], false);
        if (isset($modal_retrieve_credentials)) {
            $template .= $modal_retrieve_credentials;
        }

        return $template;
    }

    /**
     * Retrieve Account credentials detail (app key & secret)
     *
     * @throws Exception
     */
    public static function retrieveCredentials()
    {
        if (self::isPost()) {
            $id = self::getPost('id');
            $master_password = self::getPost('master_password');

            /* Perform inputs validation */
            $validation = new Validation();
            $validation->validation_rules([
                'id' => 'required',
                'master_password' => 'required'
            ]);
            $validated = $validation->run([
                'id' => $id,
                'master_password' => $master_password
            ]);
            if ($validated === false) {
                wp_send_json_error(['message' => implode('<br>', $validation->get_errors_array())]);
                wp_die();
            }

            /* Master password verify */
            $ucm_master_password = self::ucmGetOption('master_password');
            if (md5($master_password) !== $ucm_master_password) {
                wp_send_json_error(['message' => __('Master password provided is incorrect!', 'ultimate-media-on-the-cloud')]);
            } else {
                /* Fetch Amazon S3 Database Account */
                $account = PhpRockets_Model_Accounts::query([
                    'conditions' => [
                        'storage_adapter' => self::$addon_alias_key,
                        'id' => $id
                    ]
                ]);

                if ($account) {
                    $account_data = unserialize($account['value']);
                    $response_data = [
                        'app_key' => $account_data['app_key'],
                        'app_secret' => $account_data['app_secret']
                    ];
                    wp_send_json_success($response_data);
                } else {
                    wp_send_json_error(['message' => __('Account is not found.', 'ultimate-media-on-the-cloud')]);
                }
            }
        } else {
            wp_send_json_error(['message' => __('Invalid request', 'ultimate-media-on-the-cloud')]);
        }

        wp_die();
    }

    /**
     * Get file cloud storage url for media
     *
     * @param $url
     * @param $attachment_storage_meta
     * @return mixed
     */
    public function getAdapterStorageUrl ($url, $attachment_storage_meta) {
        $wp_upload = wp_upload_dir();
        $dg_root = $this->correctUploadRootDirForPush($attachment_storage_meta['path']);
        $local_upload_path = $wp_upload['baseurl'] .'/';

        $account = PhpRockets_Model_Accounts::query([
           'conditions' => [
               'id' => $attachment_storage_meta['account_id']
           ]
        ]);
        $cdn = '';
        if ($account) {
            $account_data = unserialize($account['value']);
            if ($account_data['enable_cdn']) {
                $cdn = 'cdn.';
            }
        }
        $dg_url = get_option(self::$configs->plugin_db_prefix .'option_scheme') .'://'. $attachment_storage_meta['bucket'] .'.'. $attachment_storage_meta['region'] . '.'. $cdn . $this->labels['uri'] .'/'. $dg_root;

        return str_replace($local_upload_path, $dg_url , $url);
    }

    /**
     * Get available Cloud Region Servers
     *
     * @return array
     * */
    private function getAvailableRegions()
    {
        return [
            'ams3' => 'Amsterdam',
            'sgp1' => 'Singapore',
            'sfo2' => 'San Francisco',
            'fra1' => 'Frankfurt',
            'nyc3' => 'New York',
        ];
    }

    /**
     * @param integer $id Account ID
     * @param bool    $is_ajax
     * @return array|WP_Error
     */
    public function listBuckets($id, $is_ajax = false) {
        $account = PhpRockets_Model_Accounts::query([
            'conditions' => [
                'id' => $id
            ]
        ]);
        $bucket_arr = [];
        if ($account){
            $account_data = unserialize($account['value']);
            try {
                $space = new SpacesConnect($account_data['app_key'], $account_data['app_secret'], $account_data['bucket'], $account_data['region']);
            } catch (SpacesAPIException $e) {
                if ($is_ajax) {
                    wp_send_json_error(['message' => $e->getMessage()]);
                    wp_die();
                }

                $WP_Error = new WP_Error();
                $WP_Error->add('exception', $e->getMessage());
                return $WP_Error;
            }

            $space_names = $space->ListSpaces();
            if (is_wp_error($space_names)) {
                return $space_names;
            }

            foreach ($space_names['Buckets'] as $bucket) {
                $bucket_arr[$bucket['Name']] = $bucket['Name'];
            }
        }

        return $bucket_arr;
    }

    /**
     * Update the ACL permission for Objects
     *
     * @param PhpRockets_Model_Accounts $account
     * @param array  $objects
     * @param string $bucket
     * @param string $acl
     * @return WP_Error|bool
     */
    public function updateObjectsAcl($account, $objects, $bucket, $acl = 'public-read') {
        $account_data = unserialize($account['value']);
        try {
            $space = new SpacesConnect($account_data['app_key'], $account_data['app_secret'], $bucket, $account_data['region']);
        } catch (SpacesAPIException $e) {
            $WP_Error = new WP_Error();
            $WP_Error->add('exception', $e->getMessage());
            return $WP_Error;
        }

        /* Apply updating object permissions */
        foreach ($objects as $object) {
            $result = $space->PutObjectACL($object, ['ACL' => $acl]);
            if (is_wp_error($result)) {
                return $result;
            }
        }

        return true;
    }
}